/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * callbacks.c
 * Copyright (C) Carles Sentis 2010 <realtimesunlightwallpaper.gtk@gmail.com>
 * 
 * gtk-sunlight is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gtk-sunlight is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "support.h"
#include <glib.h>
#include <glib/gstdio.h>
#include <stdlib.h>
#include <glib/gprintf.h>
#include <gdk/gdk.h>
#include <ctype.h>
//#include <libgnomeui/libgnomeui.h>

void button_sensitive (ChData *data, gboolean sensitive);
void change_wallpaper (ChData *data);
void get_and_set_options (ChData *data);
void change_wallpaper (ChData *data);

/***************************************
	Initializing on Load
***************************************/
void
on_main_window_show (GtkWidget *widget, ChData *data)
{
	gchar *filename;
	gchar *path_startup = g_strconcat(g_get_home_dir(), "/.config/autostart/", "sunlight.desktop", NULL);

	switch(data->gi_rd1)
	{
		case 1:
			switch (data->gi_rd2)
			{
				case 1:
					filename = "/usr/share/sunlight/images/mercator_small.jpg";
					gtk_toggle_button_set_active ((data->rd_mercator), TRUE);
					break;
				case 2:
					filename = "/usr/share/sunlight/images/peters_small.jpg";
					gtk_toggle_button_set_active ((data->rd_peters), TRUE);
					break;
				case 3:
					filename = "/usr/share/sunlight/images/mollweide_small.jpg";
					gtk_toggle_button_set_active ((data->rd_mollweide), TRUE);
					break;		
				case 4:
					filename = "/usr/share/sunlight/images/equirectangular_small.jpg";
					gtk_toggle_button_set_active ((data->rd_equirectangular), TRUE);
					break;
			}
			break;
		case 2:
			filename = "/usr/share/sunlight/images/dawntodusk_small.jpg";
			gtk_toggle_button_set_active ((data->rd_realistic), TRUE);
			break;
		case 3:	
			filename = "/usr/share/sunlight/images/moon_small.jpg";
			gtk_toggle_button_set_active ((data->rd_moon), TRUE);
			break;
	}
	gtk_image_set_from_file (data->main_image, filename);

	if (g_file_test(path_startup, G_FILE_TEST_EXISTS))
		gtk_toggle_button_set_active ((data->chk_start_up), TRUE);
	
	get_and_set_options(data);

    g_free(path_startup);
}

void 
on_main_window_destroy (GtkWidget *widget, ChData *data)
{
	gint time_out;	
	time_out = gtk_timeout_add(1800000, (GtkFunction) change_wallpaper, data);
}

/*******************************************************
	Window Buttons
*******************************************************/
void cmd_cancel_clicked ( GtkButton *button, ChData *data)
{
	gtk_main_quit();
}

void cmd_apply_clicked (GtkButton *button, ChData *data)
{
	change_wallpaper (data);
}

void cmd_ok_clicked ( GtkButton *button, ChData *data)
{
	gint time_out;
	gchar *contents = g_malloc(1002);
	gchar *fname = g_strconcat(g_get_home_dir(), DIR_PROJECT, "/sunlight_options", NULL);
	
	gtk_widget_hide(data->main_window);

	/*Saving gi_rd1 and gi_rd2 to a file under DIR_PROJECT*/
	snprintf(contents, 1000, "%d %d", data->gi_rd1, data->gi_rd2);
	
	if (g_file_set_contents(fname, contents, -1, NULL)){

			g_print("sunlight_options created");
				
	}else g_print("no sunlight_options file created");
		
	change_wallpaper (data);
	time_out = gtk_timeout_add(1800000, (GtkFunction) change_wallpaper, data);
	
	g_free(contents);
	g_free(fname);
}

/*******************************************************
	Changing Wallpaper Function
*******************************************************/
void
change_wallpaper (ChData *data)
{
	gchar *text1, *text2, *image_address, *set_wallpaper;
	gchar *dir_wallpapers = g_strconcat(g_get_home_dir(), DIR_WALLPAPERS, NULL);
	gchar *dir_project = g_strconcat(g_get_home_dir(), DIR_PROJECT, NULL);
	gchar *file_time;
	
	file_time = g_strdup_printf("sunlight_wallpaper%ld.jpg", time(NULL));
	text1 = "#!/bin/bash\n\nCOUNTER=0\nwhile [  $COUNTER -lt 60 ]; do\n\twget ";
	text2 = g_strdup_printf(" -O world.jpg\n\ttemp=$(stat -c%%s world.jpg)\n\t\tif [[ $temp > 1000 ]]\n\t\tthen rm %s/*.jpg\n\t\t\tmv world.jpg %s/%s\n\t\t\tbreak\n\tfi\n\tsleep 5\n\tlet COUNTER=COUNTER+1\ndone", dir_wallpapers, dir_wallpapers, file_time);
  
	switch(data->gi_rd1)
	{
		case 1:
				switch (data->gi_rd2)
				{
					case 1:
							image_address = "http://static.die.net/earth/mercator/1600.jpg";
							break;
					case 2:
							image_address = "http://static.die.net/earth/peters/1600.jpg";
							break;
					case 3:
							image_address = "http://static.die.net/earth/mollweide/1600.jpg";
							break;
					case 4:
							image_address = "http://static.die.net/earth/rectangular/1600.jpg";
							break;
				}
				break;
		case 2:
				image_address = "http://static.die.net/earth/hemisphere/1600.jpg";
				break;
		case 3:
				image_address = "http://static.die.net/moon/800.jpg";
				break;
	}
	set_wallpaper = g_strdup_printf("%s %s/%s", GCONF_SET_WALLPAPER, dir_wallpapers, file_time);
	gchar *fname = g_strconcat(dir_project, "/get_image.sh", NULL);
	gchar *contents = g_strconcat(text1, image_address, text2, NULL);
					
	if (g_file_set_contents(fname, contents, -1, NULL)) {
		if (g_chmod(fname, 0750) == 0) {
			system(fname);
			system(set_wallpaper);
		}
	} else { g_print("no get_image.sh file created"); }
	
	g_free(dir_project);
	g_free(dir_wallpapers);
	g_free(set_wallpaper);
	g_free(fname);
	g_free(contents);
	g_free(file_time);
	g_free(text2);
}

/*******************************************************
	Startup Check Button
*******************************************************/
void
chk_startup_toggled_cb (GtkToggleButton *togglebutton, ChData *data)
{
	gchar *contents = "[Desktop Entry]\nEncoding=UTF-8\nIcon=/usr/share/icons/hicolor/48x48/apps/gtk-sunlight.png\nName=Real-time Sunlight Wallpaper\nComment=World sunlight map every 30 minutes\nExec=gtk-sunlight --nodisplay\nTerminal=false\nType=Application\nCategories=Utility;\nStartupNotify=false\nHidden=false\nNoDisplay=false";
	gchar *path_startup = g_strconcat(g_get_home_dir(), "/.config/autostart/", NULL);
	gchar *path_startup_check = g_strconcat(g_get_home_dir(), "/.config/autostart/", "sunlight.desktop", NULL);

	if (gtk_toggle_button_get_active(togglebutton))
	{		
		if (!g_file_test(path_startup_check, G_FILE_TEST_EXISTS))
		{
			if (g_file_set_contents(g_strconcat(path_startup, "sunlight.desktop", NULL), contents, -1, NULL))
			{
				g_print("sunlight.desktop created");
			}
			else g_print("no sunlight.desktop file created");		
		}
	}else if (g_file_test(path_startup_check, G_FILE_TEST_EXISTS)) 
			g_remove(g_strconcat(path_startup, "sunlight.desktop", NULL));

	g_free(path_startup);
	g_free(path_startup_check);
}

/*********************************************************
	Level 1 - RadioButtons
*********************************************************/
void 
rd1_worldmap_toggled_cb (GtkToggleButton *togglebutton, ChData *data)
{
	data->gi_rd1=1;
	gchar *filename;
	
	button_sensitive (data, TRUE);

	switch (data->gi_rd2)
	{
		case 1: 
				filename = "/usr/share/sunlight/images/mercator_small.jpg";
				break;
		case 2:
				filename = "/usr/share/sunlight/images/peters_small.jpg";
				break;
		case 3:
				filename = "/usr/share/sunlight/images/mollweide_small.jpg";
				break;		
		case 4:
				filename = "/usr/share/sunlight/images/equirectangular_small.jpg";
				break;
	}
	gtk_image_set_from_file (data->main_image, filename);
}

void 
rd1_realistic_toggled_cb (GtkToggleButton *togglebutton, ChData *data)
{
	data->gi_rd1=2;
	gchar *filename = "/usr/share/sunlight/images/dawntodusk_small.jpg";
	gtk_image_set_from_file (data->main_image, filename);
	button_sensitive (data, FALSE);
}

void 
rd1_moon_toggled_cb (GtkToggleButton *togglebutton, ChData *data)
{
	data->gi_rd1=3;
	gchar *filename = "/usr/share/sunlight/images/moon_small.jpg";
	gtk_image_set_from_file (data->main_image, filename);
	button_sensitive (data, FALSE);
}

/*********************************************************
	Level 2 - RadioButtons
*********************************************************/
void 
rd2_mercator_toggled_cb (GtkToggleButton *togglebutton, ChData *data)
{
	data->gi_rd2=1;
	gchar *filename = "/usr/share/sunlight/images/mercator_small.jpg";
	gtk_image_set_from_file (data->main_image, filename);
}

void 
rd2_peters_toggled_cb (GtkToggleButton *togglebutton, ChData *data)
{
	data->gi_rd2=2;
	gchar *filename = "/usr/share/sunlight/images/peters_small.jpg";
	gtk_image_set_from_file (data->main_image, filename);
}

void 
rd2_mollweide_toggled_cb (GtkToggleButton *togglebutton, ChData *data)
{
	data->gi_rd2=3;
	gchar *filename = "/usr/share/sunlight/images/mollweide_small.jpg";
	gtk_image_set_from_file (data->main_image, filename);
}

void 
rd2_equirectangular_toggled_cb (GtkToggleButton *togglebutton, ChData *data)
{
	data->gi_rd2=4;
	gchar *filename = "/usr/share/sunlight/images/equirectangular_small.jpg";
	gtk_image_set_from_file (data->main_image, filename);
}

/*********************************************************
	ComboBoxes
*********************************************************/
void
on_cmb_style_changed (GtkComboBox *ComboBox)
{
	gchar *options;
	gint lst_option = gtk_combo_box_get_active (ComboBox);

	switch (lst_option)
	{
		case 0:	options = g_strconcat(GCONF_SET_PICTURE_OPTIONS, "wallpaper", NULL);
				break;
		case 1:	options = g_strconcat(GCONF_SET_PICTURE_OPTIONS,"zoom", NULL);
				break;
		case 2:	options = g_strconcat(GCONF_SET_PICTURE_OPTIONS,"centered", NULL);
				break;
		case 3:	options = g_strconcat(GCONF_SET_PICTURE_OPTIONS,"scaled", NULL);
				break;
		case 4:	options = g_strconcat(GCONF_SET_PICTURE_OPTIONS,"stretched", NULL);
				break;
		case 5:	options = g_strconcat(GCONF_SET_PICTURE_OPTIONS,"spanned", NULL);
				break;
	}
	system (options);
	g_free (options);
}

void
on_cmb_colors_changed (GtkComboBox *ComboBox, ChData *data)
{
	gchar *options;	
	gint lst_option = gtk_combo_box_get_active (ComboBox);

	switch (lst_option)
	{
		case 0:	options = g_strconcat(GCONF_SET_GRADIENT_OPTIONS, "solid", NULL);
				gtk_widget_hide_all (data->hbox);
				break;
		case 1:	options = g_strconcat(GCONF_SET_GRADIENT_OPTIONS,"horizontal-gradient", NULL);
				gtk_widget_show_all (data->hbox);
				break;
		case 2:	options = g_strconcat(GCONF_SET_GRADIENT_OPTIONS,"vertical-gradient", NULL);
				gtk_widget_show_all (data->hbox);
				break;	
	}
	system (options);
	g_free (options);
}

void 
on_colorbutton1_color_set (GtkColorButton *ColorButton)
{
	GdkColor color;
	gchar *options;
	
	gtk_color_button_get_color (ColorButton, &color);

	options = g_strconcat(GCONF_SET_COLOR_PRIMARY_OPTIONS, "\"", gdk_color_to_string (&color), "\"", NULL);
	system(options);
	g_free(options);
}

void 
on_colorbutton2_color_set (GtkColorButton *ColorButton)
{
	GdkColor color;
	gchar *options;
	
	gtk_color_button_get_color (ColorButton, &color);

	options = g_strconcat(GCONF_SET_COLOR_SECONDARY_OPTIONS, "\"", gdk_color_to_string (&color), "\"", NULL);
	printf("%s", options);	
	system(options);
	g_free(options);
}

/*********************************************************
	Menu Items
*********************************************************/
void 
menu_quit_activate_cb (GtkImageMenuItem *self, ChData *data)
{
	gtk_main_quit();
}

void
menu_about_activate_cb (GtkImageMenuItem *self, ChData *data)
{
	   
	static const gchar * const authors[] = {
				"Carles Sentis <realtimesunlightwallpaper.gtk@gmail.com>",
				NULL
				};

	static const gchar copyright[] = \
				"GNU GPL v2 & LGPL v2.1 \xc2\xa9 2010 Carles Sentis";

	static const gchar comments[] = "A simple but useful application which retrieve an image from the internet and set it as a wallpaper every 30 minutes.\n\nImages courtesy of die.net";

	gtk_show_about_dialog (GTK_WINDOW (data->main_window),
		       "authors", authors,
		       "comments", comments,
		       "copyright", copyright,
		       "version", "0.3.2",
		       "website", "http://realtimesunlightwallpaper.weebly.com",
		       "program-name", "Real-time Sunlight Wallpaper",
		       "logo", gdk_pixbuf_new_from_file("/usr/share/icons/hicolor/48x48/apps/gtk-sunlight.png", NULL),
		       NULL); 
}

/***********************************************************
	Misc Methods
***********************************************************/

void button_sensitive (ChData *data, gboolean sensitive)
{
	gtk_widget_set_sensitive (GTK_WIDGET (data->rd_mercator), sensitive);
	gtk_widget_set_sensitive (GTK_WIDGET (data->rd_peters), sensitive);
	gtk_widget_set_sensitive (GTK_WIDGET (data->rd_mollweide), sensitive);
	gtk_widget_set_sensitive (GTK_WIDGET (data->rd_equirectangular), sensitive);
}

void get_and_set_options (ChData *data)
{
	gchar *get_option1 = g_strconcat("gconftool-2 --get /desktop/gnome/background/picture_options > ", g_get_home_dir(), DIR_PROJECT, "/tmpfile.txt", NULL);
	gchar *get_option2 = g_strconcat("gconftool-2 --get /desktop/gnome/background/color_shading_type > ", g_get_home_dir(), DIR_PROJECT, "/tmpfile.txt", NULL);
	gchar *get_option3 = g_strconcat("gconftool-2 --get /desktop/gnome/background/primary_color > ", g_get_home_dir(), DIR_PROJECT, "/tmpfile.txt", NULL);
	gchar *get_option4 = g_strconcat("gconftool-2 --get /desktop/gnome/background/secondary_color > ", g_get_home_dir(), DIR_PROJECT, "/tmpfile.txt", NULL);

	gchar *option_file = g_strconcat(g_get_home_dir(), DIR_PROJECT, "/tmpfile.txt", NULL);
	GError     	*error = NULL;
	gchar 		*contents;
	gsize 		length;
	gchar 		*options;
	GdkColor	new_color;
	gint i;
	
	system(get_option1);

	if (g_file_test(option_file, G_FILE_TEST_EXISTS)){
		if (g_file_get_contents (option_file, &contents, &length, &error)){
			for (  i = 0; i < length; ++i )
    		{
        		if ( contents[i] == '\n' )
        	 		contents[i] = '\0';
		    }				
			if (0 == g_strcmp0 ("wallpaper", contents)){
				gtk_combo_box_set_active (data->cmb_style, 0);
			}else if (0 == g_strcmp0 ("zoom", contents)){
				gtk_combo_box_set_active (data->cmb_style, 1);
			}else if (0 == g_strcmp0 ("centered", contents)){
				gtk_combo_box_set_active (data->cmb_style, 2);
			}else if (0 == g_strcmp0 ("scaled", contents)){
				gtk_combo_box_set_active (data->cmb_style, 3);
			}else if (0 == g_strcmp0 ("stretched", contents)){
				gtk_combo_box_set_active (data->cmb_style, 4);
			}else if (0 == g_strcmp0 ("spanned", contents)){
				gtk_combo_box_set_active (data->cmb_style, 5);
			}
		}
	}
	system(get_option2);

	if (g_file_test(option_file, G_FILE_TEST_EXISTS)){
		if (g_file_get_contents (option_file, &contents, &length, &error)){
			for (  i = 0; i < length; ++i )
    		{
        		if ( contents[i] == '\n' )
        			contents[i] = '\0';
		    }			
			if (0 == g_strcmp0 ("solid", contents)){
				gtk_combo_box_set_active (data->cmb_colors, 0);
			}else if (0 == g_strcmp0 ("horizontal-gradient", contents)){
				gtk_combo_box_set_active (data->cmb_colors, 1);
			}else if (0 == g_strcmp0 ("vertical-gradient", contents)){
				gtk_combo_box_set_active (data->cmb_colors, 2);
			}
		}
	}
	system(get_option3);

	if (g_file_test(option_file, G_FILE_TEST_EXISTS)){
		if (g_file_get_contents (option_file, &contents, &length, &error)){
			for (  i = 0; i < length; ++i )
    		{
        		if ( contents[i] == '\n' )
        			contents[i] = '\0';
		    }	
			gdk_color_parse (contents, &new_color);
			gtk_color_button_set_color (data->color_picker_one, &new_color);
		}
	}

	system(get_option4);

	if (g_file_test(option_file, G_FILE_TEST_EXISTS)){
		if (g_file_get_contents (option_file, &contents, &length, &error)){
			for (  i = 0; i < length; ++i )
    		{
        		if ( contents[i] == '\n' )
        			contents[i] = '\0';
		    }	
			gdk_color_parse (contents, &new_color);
			gtk_color_button_set_color (data->color_picker_two, &new_color);
		}
	}
	g_free(option_file);
	g_free(get_option1);
	g_free(get_option2);
	g_free(get_option3);
	g_free(get_option4);
	g_free(contents);
}
