/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * sunlight.c
 * Copyright (C) Carles Sentis 2010 <realtimesunlightwallpaper.gtk@gmail.com>
 * 
 * gtk-sunlight is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gtk-sunlight is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "support.h"
#include "errno.h"
#include <glib.h>
#include <glib/gstdio.h>
#include <glib/gprintf.h>
#include <stdlib.h>
#include <gdk/gdk.h>
#include <ctype.h>
//#include <libgnomeui/libgnomeui.h>

#define UI_FILE "/usr/share/sunlight/sunlight.glade"

gboolean
make_dir(gchar *trail)
{
	gboolean success = FALSE;
	gchar *dir = g_strconcat(g_get_home_dir(), trail, NULL);

	if (!g_file_test(dir, G_FILE_TEST_IS_DIR)) {
		if (g_mkdir(dir, 0750) == 0) success = TRUE;
	} else { // exists
		success = TRUE;
	}

	g_free(dir);
	return success;
}

/* Pulling 2 integers from a string */
void
get_options (char *string, gint length, ChData *data)
{
	gint cnt = 0;

	while (1){
	
		gchar *tail;
		gint tmp_int;
		
		while (isspace (*string)) string++;
		
		if (*string == 0)
			break;

		errno = 0;
		tmp_int = strtol (string, &tail, 0);

		if (errno)
			g_printf ("Overflow\n");
		else{
			switch(cnt){
				case 0:	data->gi_rd1 = tmp_int;
				case 1: data->gi_rd2 = tmp_int;
			}cnt++;
		}
		string = tail;
	}
}

int
main( int argc, char **argv )
{
	
	
	ChData     	*data;
	GtkBuilder 	*builder;
	GError     	*error = NULL;
	gchar 		*option_file = g_strconcat(g_get_home_dir(), DIR_PROJECT, "/sunlight_options", NULL);
	gchar 		*contents;
	gsize 		length;
	gint 		time_out;

 	GOptionContext 	*context;
	gboolean 	nodisplay = FALSE;

	GOptionEntry entries[] =
	{
		{ "nodisplay", 0, 0, G_OPTION_ARG_NONE, &nodisplay, "Do not show main window", NULL },
		{ NULL }
	};

	/*GOption*/	
	if (!g_thread_supported ())
    		g_thread_init (NULL);

	context = g_option_context_new ("- Real-time Sunlight Wallpaper");
  	g_option_context_add_main_entries (context, entries, NULL);
  	g_option_context_add_group (context, gtk_get_option_group (TRUE));
	
  	if (!g_option_context_parse (context, &argc, &argv, &error)) {
		g_print ("Failed to initialize: %s\n", error->message);
    		g_error_free (error);
    		return 1;
	}
	g_option_context_free (context);
	
	/* Init GTK+ */
	gtk_init( &argc, &argv );

	/* Allocate data structure */
	data = g_slice_new( ChData );

	/* create project directories if necessary */
	if (!make_dir(DIR_PROJECT)) return 1;
	if (!make_dir(DIR_WALLPAPERS)) return 1;

	/* Check for sunlight_options file and getting gi_rd1 and gi_rd2*/
	if (g_file_test(option_file, G_FILE_TEST_EXISTS)){
		if (g_file_get_contents (option_file, &contents, &length, &error)){
			/* We are reading a string from file, and we need two integers */
			get_options(contents, length, data);		
		}else{
			data->gi_rd1=1;
			data->gi_rd2=1;
		}
		g_free(contents);
	
	}else{
		data->gi_rd1=1;
		data->gi_rd2=1;
	}
	g_free(option_file);
	
	/* Create new GtkBuilder object */
	builder = gtk_builder_new();

	if( ! gtk_builder_add_from_file( builder, UI_FILE, &error ) )
	{
		g_warning( "%s", error->message );
		g_free( error );
		return( 1 );
	}

	/* Get objects from UI */
#define GW( name ) CH_GET_WIDGET( builder, name, data )
	GW( main_window );
#undef GW

	/* get the widgets which will be referenced in callbacks */
	data->main_window = GTK_WIDGET (gtk_builder_get_object (builder, "main_window"));
	data->main_image = GTK_IMAGE (gtk_builder_get_object (builder, "sunlight_image"));
	data->rd_worldmap = GTK_TOGGLE_BUTTON(gtk_builder_get_object (builder, "rd1_worldmap"));
	data->rd_mercator = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "rd2_mercator"));
	data->rd_peters = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "rd2_peters"));
	data->rd_mollweide = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "rd2_mollweide"));
	data->rd_equirectangular = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "rd2_equirectangular"));
	data->rd_realistic = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "rd1_realistic"));
	data->rd_moon = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "rd1_moon"));
	data->chk_start_up = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "chk_startup"));
	data->cmb_style = 	GTK_COMBO_BOX (gtk_builder_get_object (builder, "cmb_style"));
	data->cmb_colors = 	GTK_COMBO_BOX (gtk_builder_get_object (builder, "cmb_colors"));
	data->color_picker_one = GTK_COLOR_BUTTON (gtk_builder_get_object (builder, "colorbutton1"));
	data->color_picker_two = GTK_COLOR_BUTTON (gtk_builder_get_object (builder, "colorbutton2"));
	data->hbox = GTK_WIDGET (gtk_builder_get_object (builder, "hbox4"));

	/* Connect signals */
	gtk_builder_connect_signals( builder, data );

	/* Destroy builder, since we don't need it anymore */
	g_object_unref( G_OBJECT( builder ) );
	
	gchar *kill_sunlight = "killall gtk-sunlight -o 5s";
	system (kill_sunlight);

	if (!nodisplay) gtk_widget_show( data->main_window );
	else{
		change_wallpaper (data);
		time_out = gtk_timeout_add(1800000, (GtkFunction) change_wallpaper, data);
	}
	
	/* Start main loop */
	gtk_main();

	/* Free any allocated data */
	g_slice_free( ChData, data );


	return( 0 );
}




